#include "avr-io.h"
#include "sig-avr.h"
#include "MotorTask.h"

//+---------------------------------------------------------
// SIGNAL(SIG_INTERRUPT0) & SIGNAL(SIG_INTERRUPT1)
//
// INTERRUPT0 and 1 are used to time the R/C pulse inputs.
//
// The general algorithm is to grab the system timer (8uS/tick) on
// the leading and trailing edge of the pulse.  The code flips the
// sense of the interrupt with each edge.  The bit flag "done" is used to
// tell the overlying task to calculate the new value.
//
// NB: This is NOT an AVRX interrupt handler!  Use the GCC SIGNAL() mechanism.
// Eventually AvrX needs to be modified to not disable these interrupts during
// a contex switch as the current latency is > 8uS @ 8mhz (gives a 1 count jitter)
//
SIGNAL(SIG_INTERRUPT0)
{
    unsigned char timer = inp(TCNT0);
    unsigned char mcucr = inp(MCUCR);
    
    if (mcucr & BV(ISC00))
    {
        outp(mcucr & ~BV(ISC00), MCUCR);
        EndCritical();
        RC0.start = timer;
    }
    else
    {
        outp(mcucr | BV(ISC00), MCUCR);
        EndCritical();
        RC0.end = timer;
        RC0.done = TRUE;
    }
}

SIGNAL(SIG_INTERRUPT1)
{
    unsigned char timer = inp(TCNT0);   // the use of temps creates significantly 
    unsigned char mcucr = inp(MCUCR);   // better code.
    
    if (mcucr & BV(ISC10))
    {
        outp(mcucr & ~BV(ISC10), MCUCR);
        EndCritical();
        RC1.start = timer;
    }
    else
    {
        outp(mcucr | BV(ISC10), MCUCR);
        EndCritical();
        RC1.end = timer;
        RC1.done = TRUE;
    }
}

SIGNAL(SIG_COMPARATOR)
{
    unsigned char timer = inp(TCNT0);   // the use of temps creates significantly 
    
    if (inp(ACSR) & BV(ACIS0))
    {
        cbi(ACSR, ACIS0);
        EndCritical();
        RC2.start = timer;
    }
    else
    {
        sbi(ACSR, ACIS0);
        EndCritical();
        RC2.end = timer;
        RC2.done = TRUE;
    }
}


