/*
mob.c

Copyright 2001 Larry Barello

Author: Larry Barello
        larry@barello.net

 OSMC master CPU control program.
 
-*/
#define USE_DEBUG_MONITOR

#include "avrx-io.h"
#include "avrx-signal.h"
#include "eeprom.h"
#include "avrx.h"
#include "hardware.h"
#include "mob_eeprom.h"

// Declare external tasks

AVRX_EXTERNTASK(MotorTask);
AVRX_EXTERNTASK(LcdTask);
//AVRX_EXTERNTASK(I2CTask);
AVRX_EXTERNTASK(LedTask);

#if defined(USE_DEBUG_MONITOR)
    void InitSerialIO(unsigned char);     // From Avrx.a (debug monitor)
    AVRX_GCC_TASK(Monitor, 20, 0);
#else
    AVRX_EXTERNTASK(SerialTask);
#endif


/*+-----------------------------------------------

AvrX system timer

This timer is free running so the tickrate will be some odd
fraction of the CPU clock (8mHz/64/256 = 488 hz, or 2.048mS/tick)
*/
AVRX_SIGINT(SIG_OVERFLOW0)
{
    IntProlog();
    EndCritical();
    AvrXTimerHandler();
    Epilog();
}

// Main startup

int main(void)
{
    AvrXSetKernelStack(0);      // Take what GCC gives us

    ReadEEPROMandInitModes();

    outp(MCUCR_INIT, MCUCR);    // See harware.h
    outp(TCCR0_INIT, TCCR0);
    outp(TIMSK_INIT, TIMSK);
    outp(GIMSK_INIT, GIMSK);
    outp(ACSR_INIT, ACSR);

    outp(DDRA_INIT, DDRA);      // See hardware.h for initial
    outp(PORTA_INIT, PORTA);    // I/O port definitions
    outp(DDRB_INIT, DDRB);
    outp(PORTB_INIT, PORTB);
    outp(DDRC_INIT, DDRC);
    outp(PORTC_INIT, PORTC);
    outp(DDRD_INIT, DDRD);
    outp(PORTD_INIT, PORTD);
    
    // Initialize and run tasks
    
#if defined(USE_DEBUG_MONITOR)
   InitSerialIO(UBRR_INIT);
    AvrXRunTask(TCB(Monitor));
#else
    AvrXRunTask(TCB(SerialTask));
#endif
    AvrXRunTask(TCB(MotorTask));
    AvrXRunTask(TCB(LcdTask));
//    AvrXRunTask(TCB(I2CTask));
    AvrXRunTask(TCB(LedTask));

    Epilog();

    return 0;
}


